<?php
Anc_Application::autoload('Anc_Library_Plugin_Publisher');

class Anc_Publisher_Wordpress extends Anc_Library_Plugin_Publisher
{
    protected $_publicationDelay = 0;  //removed in 2.0
    protected $_publicationStartTime = 36000; //10:00
    protected $_publicationRange = 28800; //8 hours

    protected $_publicationDeltaRatio = 0.25;  // Random post offset +/- as a part of post step

    protected $_addMoreTag = TRUE;

    protected $_publication;

    // Init options
    protected $_scheduleSettings;
    protected $_publishType;
    protected $_postCategories = array();

    public function init()
    {
        global $wpdb;

        // Workaround for connection timeout on HostGator
        $wpdb->query("set session wait_timeout=600");
    }

    public function publish(array $merchant)
    {
        $text = $this->_htmlEscape($merchant['text']);
        if ($this->_addMoreTag) {
            $paragraphs = explode("\n\n", $text);
            if (count($paragraphs) > 2) {
                $text = '';
                $last = count($paragraphs) - 1;
                $i = 0;
                foreach($paragraphs as $paragraph) {
                    $text .= $paragraph;
                    if (1 == $i) {
                        $text .= "\n<!--more-->\n";
                    } elseif ($i < $last) {
                        $text .= "\n\n";
                    }
                    $i++;
                }
            }
        }

        $post = array();
        $post['post_content'] = $text;
        $post['post_title'] = $merchant['title'];
        if (isset($merchant['description'])) {
            $post['post_excerpt'] = $merchant['description'];
        }
        if (isset($merchant['keywords'])) {
            $post['tags_input'] = implode(', ', $merchant['keywords']);
        }
        $post['post_category'] = $this->_postCategories;

        $scheduleTime = $this->_getTime();
        if (!is_null($scheduleTime)) {
            $post['post_status'] = 'future';
            $post['post_date'] = date('Y-m-d H:i:s', $scheduleTime);
        } else {
            $post['post_status'] = 'publish';
        }

        // Write post
        $postId = wp_insert_post($post);
        if (is_a($postId, 'WP_Error')) {
            throw new Exception($postId->get_error_message());
        } else if (!$postId) {
            global $wpdb;
            if ('' != $wpdb->last_error) {
                throw new Exception('Post creation failed due to database error: ' . $wpdb->last_error);
            }
            throw new Exception('Post creation failed due to unknown error.');
        }

        // Save metadata
        add_post_meta($postId, 'cbc_version', Cbc_WpPlugin::PLUGIN_VERSION);
        add_post_meta($postId, 'cbc_merchant_id', $merchant['id']);
        add_post_meta($postId, 'cbc_merchant_url', $merchant['url']);
        ///add_post_meta($postId, 'cbc_affiliate_url', $merchant['affiliateNetwork-url']);

        //@todo: move this to sepate pulisher
        if ($this->_app->getPingThumbs()
            && (is_null($scheduleTime) || $scheduleTime <= strtotime('+5 days'))
        ) {
            // send query to page peeker
            require_once 'Pumka/Thumbs/PagePeeker.php';
            $thumb = new Pumka_Thumbs_PagePeeker;
            $thumb->setOptions(array(
                'size'  => $this->_app->getThumbSize(),
                'url'   => $merchant['url'],
            ));
            try {
                $thumb->downloadImage();
            } catch (Pumka_Thumbs_PagePeeker_Exception $e) {
                // Ignore errors so far
            }
        }

        //Ping feedback
        $this->_app->feedback();
    }

    public function isPublished($merchantId)
    {
        global $wpdb;
        $sql = $wpdb->prepare("SELECT COUNT(*) FROM `{$wpdb->posts}` p JOIN `{$wpdb->postmeta}` m ON p.`ID`=m.`post_id` WHERE p.`post_status`!='trash' AND m.`meta_key`='cbc_merchant_id' AND m.`meta_value`=%s",
            $merchantId);
        $result = $wpdb->get_var($sql);
        if ('' != $wpdb->last_error) {
            throw new Exception($wpdb->last_error);
        }

        return !empty($result);
    }

    protected function _htmlEscape($text)
    {
        $text = (string) $text;
        if ('' === $text) return '';

        $result = @htmlspecialchars($text, ENT_COMPAT, 'UTF-8');
        if (empty($result)) {
            $result = @htmlspecialchars(utf8_encode($text), ENT_COMPAT, 'UTF-8');
        }

        return $result;
    }

    protected function _getTime()
    {
        if ('immediately' == $this->_publishType) {
            return NULL;
        }

        if (!$this->_publication) {
            $start = time() + $this->_publicationDelay;
            $day = floor($start/86400);
            $time = $start % 86400;
            //$step = $this->_scheduleSettings['avg'] > 1
            //    ? $this->_publicationRange / ($this->_scheduleSettings['avg'] - 1)
            //    : 0;

            $todaysPosts = $this->_scheduleSettings['avg'];
            $endTime = $this->_publicationStartTime + $this->_publicationRange;
            $weekSchedule = $this->_weekSchedule();
            if (!(($todaysPosts > 1) && (
                ($time>=$this->_publicationStartTime && $time<$endTime)
                || ($endTime>=86400 && $time<($endTime-86400)))))
            {
                // Nothing to publish today. Move to tomorrow
                $this->_publication = array(
                    'day' => $day+1,
                    'todaysPosts' => $weekSchedule[0],
                    'postCounter' => 0,
                    'dayCounter' => 0,
                    'weekSchedule' => $weekSchedule,
                );
                return NULL;
            }

            // If posting overnight:
            if ($endTime>=86400 && $time<($endTime-86400)) {
                $time += 86400;
                $day--;
            }
            $step = $this->_publicationRange / ($todaysPosts);
            $counter = floor(($time - $this->_publicationStartTime)/$step);

            $this->_publication = array(
                'day' => $day,
                'todaysPosts' => $todaysPosts,
                'postCounter' => $counter,
                'dayCounter' => -1,
                'weekSchedule' => $weekSchedule,
            );

            return NULL;
        }

        $postCounter = $this->_publication['postCounter'];
        if ($postCounter >= $this->_publication['todaysPosts']) {
            while (true) {
                //Move to next day
                $this->_publication['day']++;
                $dayCounter = ++$this->_publication['dayCounter'];
                if ($dayCounter >= 7) {
                    $dayCounter = $this->_publication['dayCounter'] = 0;
                    $this->_publication['weekSchedule'] = $this->_weekSchedule();
                }
                $todaysPosts = $this->_publication['weekSchedule'][$dayCounter];
                if ($todaysPosts > 0) {
                    $this->_publication['todaysPosts'] = $todaysPosts;
                    $postCounter = $this->_publication['postCounter'] = 0;
                    break;
                }
            }
        }

        $day = $this->_publication['day'];
        $todaysPosts = $this->_publication['todaysPosts'];

        if ($todaysPosts > 1) {
            // Set first post time
            if (0 == $postCounter || !isset($this->_publication['step'])) {
                $this->_publication['step'] = $this->_publicationRange / $todaysPosts;
            }
            $step = $this->_publication['step'];
            $delta = (int)($step * $this->_publicationDeltaRatio);
            $result = $step * ($postCounter + 0.5) + rand(-$delta, $delta);
        } else {
            // 1 post per day: any time whithin publication range
            $result = rand(0, $this->_publicationRange);
        }
        $this->_publication['postCounter']++;

        $result += $this->_publicationStartTime + (86400 * $day);
        if ($result < time()) {
            return NULL;
        }

        return $result;
    }

    protected function _weekSchedule()
    {
        $total = $this->_scheduleSettings['weekly'];
        $min = $this->_scheduleSettings['min'];
        $max = $this->_scheduleSettings['max'];

        $lottery = array();
        for ($i = 0; $i < 7; $i++) {
            $lottery[$i] = array(
                'index' => $i,
                'count' => $min,
            );
        }
        $result = array();

        $count = $total - 7*$min;
        for ($i = 0; $i < $count; $i++) {
            $item = rand(0, count($lottery)-1);
            $lottery[$item]['count']++;
            if ($lottery[$item]['count'] >= $max) {
                $result[$lottery[$item]['index']] = $max;
                unset($lottery[$item]);
                // Reindex array
                $lottery = array_values($lottery);
            }
        }

        foreach ($lottery as $val) {
            $result[$val['index']] = $val['count'];
        }

        return $result;
    }

    //
    // Getters & setters
    //

    public function setScheduleSettings($value)
    {
        $this->_scheduleSettings = $value;
    }

    public function setPublishType($value)
    {
        $this->_publishType = $value;
    }

    public function getPostsPerDay()
    {
        return $this->_postsPerDay;
    }

    public function setPostCategories($value)
    {
        $this->_postCategories = $value;
    }

    public function getPostCategories()
    {
        return $this->_postCategories;
    }
}