<?php
Anc_Application::autoload('Anc_Library_Plugin_Scraper');

class Anc_Scraper_Webpage extends Anc_Library_Plugin_Scraper
{
    protected $_useAnonimizer = FALSE;
    protected $_maxSize = 2097152;

    protected $_textRemoveTags = array('script', 'style', 'object', 'video');
    protected $_textStripTags = array('a', 'i', 'em', 'b', 'strong', 's', 'u',
        'font', 'center', 'left', 'right', 'span');
    protected $_textContainTags = array('p', 'div', 'td');
    protected $_textInvalidParagraphPattern = '/(?:\||©|•|menu|copyright|»|\b[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}\b)/i';
    protected $_textParagraphMinChars = 100;

    protected $_textMinWords = 100;
    protected $_textMaxWords = 150;

    protected $_titleMinWords = NULL;
    protected $_titleMaxWords = 30;

    protected $_keywordMaxLength = 30;
    protected $_maxKeywords = 10;

    protected $_http;

    public function init()
    {
        parent::init();

        Anc_Application::autoload('Anc_Library_Http_ClientWrapper');
        $this->_http = new Anc_Library_Http_ClientWrapper;
        $this->_http->setMaxSize($this->_maxSize);
    }

    public function scrape(array &$merchant)
    {
        usleep(500);

        //Allows processing of large texts
        //ini_set('pcre.backtrack_limit', 100000);
        //ini_set('pcre.recursion_limit', 100000);

        $url = $merchant['url'];

        try {
            //$html = $this->_http->get('http://anonymouse.org/cgi-bin/anon-www.cgi/' . $url);
            $html = $this->_http->get($this->_useAnonimizer
                ? 'http://anonymouse.org/cgi-bin/anon-www.cgi/' . $url
                : $url);
        } catch (Anc_Library_Http_Exception $e) {
            //@todo: Error handling
            $this->_throwError("Cannot open URL {$url}: {$e->getMessage()}");
        }

        //Replace merchant URL with response URL
        $merchant['url'] = $this->_http->getUrl(TRUE);

        //unset($this->_http);

        if ($this->_useAnonimizer) {
            $html = str_replace('[Anonymoused]', '', $html);
        }

        //Remove comments
        $this->_eraseFragments($html, '<!--', '-->');

        //Normalize spaces and whitespace
        $this->_massReplace(array("\n", "\r", "\t", '&nbsp;', '&#160;'),
            ' ', $html);

        //Remove control chars
        $this->_massReplace(
            array(
                chr(0), chr(1), chr(2), chr(3), chr(4), chr(5), chr(6), chr(7),
                chr(8), chr(9), chr(10), chr(11), chr(12), chr(13), chr(14),
                chr(15), chr(16), chr(17), chr(18), chr(19), chr(20), chr(21),
                chr(22), chr(23), chr(24), chr(25), chr(26), chr(27), chr(28),
                chr(29), chr(30), chr(31),

                chr(194) . chr(128), chr(194) . chr(129), chr(194) . chr(130),
                chr(194) . chr(131), chr(194) . chr(132), chr(194) . chr(133),
                chr(194) . chr(134), chr(194) . chr(135), chr(194) . chr(136),
                chr(194) . chr(137), chr(194) . chr(138), chr(194) . chr(139),
                chr(194) . chr(140), chr(194) . chr(141), chr(194) . chr(142),
                chr(194) . chr(143), chr(194) . chr(144), chr(194) . chr(145),
                chr(194) . chr(146), chr(194) . chr(147), chr(194) . chr(148),
                chr(194) . chr(149), chr(194) . chr(150), chr(194) . chr(151),
                chr(194) . chr(152), chr(194) . chr(153), chr(194) . chr(154),
                chr(194) . chr(155), chr(194) . chr(156), chr(194) . chr(157),
                chr(194) . chr(158), chr(194) . chr(159),
            ),
            '', $html);

        //Get title

        if (preg_match('/<title\b[^>]*>(.*?)<\/title>/is', $html, $matches)) {
            $title = trim($matches[1]);
            if ('' == $title) {
                $this->_throwError('Title is empty');
            }

            $title = $this->_filterText($title);
            $title = $this->_htmlDecode($title);

            if ('ERROR:' == strtoupper(substr($title, 0, 6))) {
                $this->_throwError(trim($title));
            }

            $title = str_replace('|', '-', $title);
            $title = $this->_normalizeWhiteSpace($title);
            $title = $this->_limitWordCount($title, $this->_titleMinWords,
                $this->_titleMaxWords);

            $merchant['title'] = $title;
        } else {
            $this->_throwError('Title not found');
        }

        //Get descrition
        if (preg_match('/<meta\b\s+name="description"\s+content="(.*?)"/is', $html, $matches)) {
            $description = trim($matches[1]);
            if ('' != $description) {
                $description = $this->_htmlDecode($description);
                $description = $this->_normalizeWhiteSpace($description);
                $merchant['description'] = $description;
            }
        } else {
            //DebugBreak();
        }

        //Get keywords
        if (preg_match('/<meta\b\s+name="keywords"\s+content="(.*?)"/is', $html, $matches)) {
            $keywords = trim($matches[1]);

            if ('' != $keywords) {
                $keywords = explode(',', $this->_htmlDecode($matches[1]));
                $resultKeywords = array();
                foreach($keywords as $i=>$keyword) {
                    $keyword = $this->_normalizeWhiteSpace($keyword);
                    if (!empty($keyword) && strlen($keyword) <= $this->_keywordMaxLength) {
                        $resultKeywords[] = $keyword;
                    }
                    if (count($resultKeywords) >= $this->_maxKeywords) {
                        break;
                    }
                }
                $merchant['keywords'] = $resultKeywords;
            }
        } else {
            //DebugBreak();
        }

        // Extract body
        if (preg_match('/<body\b[^>]*>(.+?)<\/body>/is', $html, $matches)) {
            $html = $matches[1];
        }

        /*
        //Other cleanup (causes faults?)
        $html = preg_replace('/<\!--.*?-->/sm', '', $html);
        $html = preg_replace('/<\!\[CDATA\[.*?\]\]/ism', '', $html);
        //Remove unwanted tags
        foreach ($this->_textRemoveTags as $tag) {
            $html = preg_replace("/<{$tag}\b[^>]*>.*?<\/{$tag}>/ism", ' ', $html);
        }
        */
        //Remove line breaks
        $html = preg_replace('/<br\b.*?>/is', ' ', $html);
        //Strip some allowed tags
        $html = preg_replace('/<(?:\/)?(' . implode('|', $this->_textStripTags) . ')\b[^>]*>/is', '', $html);

        if (!preg_match_all('/<(' . implode('|', $this->_textContainTags) .')\b[^>]*>([^<>]+)<\/\1>/is', $html, $matches))
        {
            $this->_throwError('Cannot extract any meaningful text from the landing page.');
        }

        $text = array();
        $wordCount = 0;
        foreach($matches[2] as $paragraph) {
            $paragraph = trim($paragraph);
            if (strlen($paragraph) < $this->_textParagraphMinChars) {
                continue;
            }

            //Strip white space
            //$paragraph = trim(preg_replace('/\s*\"\s*/sm', '"', $paragraph));

            $paragraph = $this->_normalizeWhiteSpace($paragraph);

            //$paragraph = trim(str_replace('[BR]', "\n", $paragraph));
            //$paragraph = trim(preg_replace('/\s*\n+\s*/sm', "\n", $paragraph));
            if (strlen($paragraph) < $this->_textParagraphMinChars) {
                continue;
            }
            if (strtoupper($paragraph) == $paragraph) {
                //Skip all caps
                continue;
            }

            if (preg_match_all('/[A-Z]/', $paragraph, $dummy) > 0.50*strlen($paragraph)) {
                //Too many caps
                continue;
            }

            $paragraph = $this->_htmlDecode(trim($paragraph));
            if (preg_match($this->_textInvalidParagraphPattern, $paragraph)) {
                continue;
            }

            $text[] = $paragraph;

            $wordCount += str_word_count($paragraph);
            if ($wordCount > $this->_textMaxWords) {
                break;
            }
        }

        if ($text) {
            $text = implode("\n\n", $text);
            $text = $this->_limitWordCount($text, $this->_textMinWords,
                $this->_textMaxWords); // May return empty text if not enough words
        } else {
            $text = '';
        }

        if ('' == $text) {
            $this->_throwError('Cannot extract any meaningful text from the landing page.');
        }

        $merchant['text'] = $text;
    }

    protected function _limitWordCount($text, $minLimit=NULL, $maxLimit=NULL)
    {
        // Null limit means unlimited
        if (is_null($minLimit) && is_null($maxLimit)) {
            return $text;
        }

        $text = wp_check_invalid_utf8($text, TRUE);

        $words = preg_split('~[^\p{L}\p{N}\']+~u', $text, -1,
            PREG_SPLIT_OFFSET_CAPTURE|PREG_SPLIT_NO_EMPTY);

        $count = count($words);

        if (!is_null($minLimit) && $count < $minLimit) {
            return '';
        }

        if (is_null($maxLimit) || $count <= $maxLimit) {
            return $text;
        }

        $last = $words[$maxLimit-1];
        $text = substr($text, 0, $last[1] + strlen($last[0])) . '...';

        return $text;
    }

    protected function _htmlDecode($text)
    {
        $result = html_entity_decode($text, ENT_QUOTES, 'UTF-8');
        if (empty($result)) {
            $result = html_entity_decode($text, ENT_QUOTES);
        }

        return $result;
    }

    protected function _eraseFragments(&$subject, $start, $end)
    {
        for ($i=0; $i<200; $i++) {
            $posStart = strpos($subject, $start);
            if ($posStart===FALSE) break;
            $pos = $posStart+strlen($start);
            if ($pos >= strlen($subject)) {
                break;
            }
            $posEnd = strpos($subject, $end, $pos);
            if ($posEnd===FALSE) break;

            $subject = substr($subject, 0, $posStart) . substr($subject, $posEnd + strlen($end));
        }
    }

    protected function _filterText($text)
    {
        $text = preg_replace('/<\s*(\w+)\b[^>]*>.*?<\s*\/\1\s*>/is', '', $text);
        $text = strip_tags($text);

        return $text;
    }

    protected function _massReplace(array $search, $replace, &$subject)
    {
        foreach ($search as $s) {
            $subject = str_replace($s, $replace, $subject);
        }
    }

    protected function _normalizeWhiteSpace($text)
    {
        $result = trim(preg_replace('/[\x{0009}\x{000A}\x{0010}\x{000D}]+/usm', ' ', $text));
        if (preg_last_error()) {
            $result = trim(preg_replace('/\s+/sm', ' ', $text));
        }

        return $result;
    }

    //
    // Getters & setters
    //

    public function setTextMaxWords($value)
    {
        $this->_textMaxWords = $value;
    }
}