<?php
require_once CBC_WPMODULE_PATH . '/controllers/Abstract/Admin.php';

class Cbc_WpModule_Controller_AdminController
    extends Cbc_WpModule_Controller_Abstract_Admin
{
    public function indexAction()
    {
        /**
        * @var Cbc_WpModule_Model_CoreApp
        */
        $core = Cbc_WpModule_Application::getModel('CoreApp');

        if ($core->isActive()) {
            $this->_displayProgress();
            return;
        }

        /**
        * @var Cbc_WpModule_Model_Options
        */
        $options = Cbc_WpModule_Application::getModel('Options');

        /**
        * @var Cbc_WpModule_Form_Main
        */
        $form = Cbc_WpModule_Application::getForm('Admin');

        $form->loadDefaults();

        $categories = array();
        if ($form->hasCredentials() || $_POST) {
            try {
                $categories = $core->getCategories($form->extractCredentials($_POST? $_POST : NULL));
            } catch (Cbc_WpModule_Model_CoreAppException $e) {
                $this->_addMessage($this->_view->escape($e->getMessage()),
                    'error');
            }

            if ($categories) {
                $form->setClickbankCategories($categories);
            }
        }

        $action = NULL;

        if ($_POST) {
            if (isset($_POST['action'])) {
                $action = $_POST['action'];
            }

            if (!$form->isValid($_POST)) {
                foreach ($form->getElements() as $element) {
                    foreach ($element->getMessages() as $message) {
                        $this->_addMessage(
                            $this->_view->escape("{$element->getLabel()}: {$message}"),
                            'error');
                    }
                }

                if ('add_selected' == $action) {
                    $this->_view->items = $_SESSION[Cbc_WpPlugin::PREFIX . '_preview_items'];
                }

            } else {
                $form->saveDefaults();

                $data = $form->getValues();

                $formOptions = array(
                    'main' => array(
                        'maxMerchants' => $data['option_max_posts'],
                    ),
                    'adapter' => array(
                        'affiliateId' => $data['clickbank_id'],
                        'password' => $data['clickbank_password'],
                        'trackingId' => $data['tracking_id'],
                    ),
                );

                if ('add_selected' == $action) {
                    $items = $_SESSION[Cbc_WpPlugin::PREFIX . '_preview_items'];
                    $selectedItems = $data['select_items'];
                    foreach ($items as $key=>$item) {
                        if ($item['published']
                            || !in_array($key, $selectedItems)
                        ) {
                            unset($items[$key]);
                        }
                    }
                    $formOptions['main']['merchantList'] = $items;
                } else {
                    $formOptions['adapter']['categoryId'] = $data['clickbank_category'];
                    $formOptions['adapter']['subcategoryId'] = $form->getSubcategory();
                    $formOptions['adapter']['sortOrder'] = $data['sort_order'];
                    $formOptions['adapter']['sortReverse'] = $data['sort_direction'];
                    $formOptions['adapter']['searchLanguage'] = $data['search_language'];
                    $formOptions['adapter']['searchIncludeKeywords'] = isset($data['search_include_keywords'])
                        ? explode("\n", $data['search_include_keywords'])
                        : NULL;
                    $formOptions['adapter']['searchExcludeKeywords'] = isset($data['search_exclude_keywords'])
                        ? explode("\n", $data['search_exclude_keywords'])
                        : NULL;
                }

                if ('preview' == $action) {
                    try {
                        $items = $core->preview($formOptions);
                        if ($items) {
                            $this->_view->items = $items;
                        } else {
                            $this->_addMessage('No products found for the search criteria.');
                        }
                        $_SESSION[Cbc_WpPlugin::PREFIX . '_preview_items'] = $items;

                    } catch (Anc_Library_Plugin_Exception $e) {
                        $this->_addMessage($this->_view->escape($e->getMessage()), 'error');
                    }
                } else {
                    $formOptions['postsPerWeek'] = $data['posts_per_week'];
                    $formOptions['publishType'] = $data['publish_type'];
                    $formOptions['publisher'] = array(
                        'postCategories' => $data['post_category']
                    );
                    $formOptions['scraper'] = array(
                        'textMaxWords' => $data['option_max_words']
                    );

                    $this->_view->startOptions = $formOptions;

                    @session_regenerate_id();

                    $this->_displayProgress();
                    return;
                }
            }
        }

        $this->_view->form = $form;
        $this->_view->categories = $categories;
        $this->_view->action = $action;

        $this->_view->layout->announcement = Cbc_WpModule_Application::getModel('AnnouncementHtml')->get();
    }

    public function previewAction()
    {
        $categoryId = $this->_getPreviewField('clickbank_category');
        if (!is_null($categoryId)) {
            $subcategoryId = $this->_getPreviewField('clickbank_subcategory_' . $categoryId);
        } else {
            $subcategoryId = NULL;
        }

        try {
            $url = Cbc_WpModule_Application::getModel('Search')->composeUrl(array(
                'clickbankId' => $this->_getPreviewField('clickbank_id'),
                'categoryId' => $categoryId,
                'subcategoryId' => $subcategoryId,
                'sortOrder' => $this->_getPreviewField('sort_order'),
                'sortReverse' => ('DESC' == $this->_getPreviewField('sort_direction')),
                'searchLanguage' => $this->_getPreviewField('search_language'),
                'searchIncludeKeywords' => $this->_getPreviewKeywordsField('search_include_keywords'),
                'searchExcludeKeywords' => $this->_getPreviewKeywordsField('search_exclude_keywords'),
            ));
        } catch (Cbc_WpModule_Model_Search_Exception $e) {
            wp_die($e->getMessage());
        }

        $this->_redirectUrl($url);
    }

    protected function _getPreviewField($name)
    {
        $result = NULL;
        if (isset($_POST[$name])) {
            $val = trim($_POST[$name]);
            if ('' != $val) {
                $result = $val;
            }
        }

        return $result;
    }

    protected function _getPreviewKeywordsField($name)
    {
        $result = $this->_getPreviewField($name);
        if (!is_null($result)) {
            $result = explode("\n", $result);
            foreach ($result as $key => $val) {
                $val = trim($val);
                if ('' == $val) {
                    unset($result[$key]);
                } else {
                    $result[$key] = $val;
                }
            }
        }

        return $result? array_values($result): NULL;
    }

    protected function _addFormField(array &$formData, $name, $options=array())
    {
        $formData[$name] = $options;
    }

    public function headAction()
    {
        $this->_layoutScript = NULL;
    }

    protected function _displayProgress()
    {
        $this->_view->updateTimeout = 3000;
        $this->_viewScript = 'progress/index.phtml';
    }

    public function optionsAction()
    {
        /**
        * @var Cbc_WpModule_Form_Main
        */
        $form = Cbc_WpModule_Application::getForm('Options');

        if (count($_POST)) {
            if (!$form->isValid($_POST)) {
                foreach ($form->getElements() as $element) {
                    foreach ($element->getMessages() as $message) {
                        $this->_addMessage(
                            $this->_view->escape("{$element->getLabel()}: {$message}"),
                            'error');
                    }
                }
            } else {
                try {
                    $form->saveValues();
                    $this->_addMessage('Settings saved');
                } catch (Cbc_WpModule_Form_Options_Exception $e) {
                    $message = $e->getMessage();
                    if (($element = $e->getElement())
                        && ($element = $form->getElement($element))
                    ) {
                        $message = "{$element->getLabel()}: {$message}";
                    }

                    $this->_addMessage(
                        $this->_view->escape($message),
                        'error');
                }
            }
        }

        $this->_view->form = $form;
    }
}