<?php
require_once 'CbcZend/Form.php';

require_once 'Pumka/Filter/StripSlashes.php';

class Cbc_WpModule_Form_Admin extends CbcZend_Form
{
    protected $_clickbankCategories = array();

    protected $_credentialFields = array(
        'clickbank_id',
        'clickbank_password',
    );

    public function init()
    {
        /**
        * @var Cbc_WpModule_Model_CoreApp
        */
        $core = Cbc_WpModule_Application::getModel('CoreApp');
        $multiOptions = $core->getMultiOptions();
        $constraints = $core->getConstraints();

        require_once 'Pumka/Filter/KeywordList.php';

        $this->setDisableTranslator(TRUE);

        $this->addElement('text', 'clickbank_id', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                'StringToLower',
            ),
            'validators' => array(
                array('Regex', TRUE, array(
                    'pattern' => '~^[a-z0-9]+$~u',
                    'messages' => array(
                        'regexNotMatch' => "'%value%' contains invalid characters",
                    ),
                )),
            ),
            'label' => 'ClickBank ID',
        ));

        $this->addElement('password', 'clickbank_password', array(
            'required' => TRUE,
            'filters' => array(
                new Pumka_Filter_StripSlashes,
                'StripNewlines',
            ),
            'label' => 'ClickBank Password'
        ));

        $this->addElement('text', 'tracking_id', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                array('Null', array('string')),
            ),
            'validators' => array(
                array('StringLength', TRUE, array(
                    'max' => 24,
                )),
            ),
            'label' => 'Append this tracking ID to my affiliate links',
        ));

        $this->addElement('select', 'clickbank_category', array(
            //'required' => TRUE,
            'filters' => array(
                'Null',
            ),
            'label' => 'ClickBank Category',
            'disabled' => TRUE,
        ));

        $this->addElement('radio', 'publish_type', array(
            'required' => TRUE,
            'multiOptions' => array(
                'schedule' => 'Schedule',
                'immediately' => 'Publish all posts immediately',
            ),
            'label' => 'Scheduling options:',
        ));

        $this->addElement('select', 'posts_per_week', array(
            'multiOptions' => $multiOptions['postsPerWeek'],
            'required' => TRUE,
            'label' => 'Schedule posts per week',
        ));

        $this->addElement('multiCheckbox', 'post_category', array(
            'label' => 'Categories',
        ));

        $elem = $this->getElement('post_category');
        foreach(get_categories(array('hide_empty'=>0)) as $category) {
            $elem->addMultiOption($category->term_id, $category->name);
        }

        $this->addElement('text', 'option_max_posts', array(
            'required' => TRUE,
            'filters' => array(
                'Digits',
            ),
            'validators' => array(
                'Digits',
                array('Between', TRUE, array(
                    'min' => $constraints['maxMerchants']['min'],
                    'max' => $constraints['maxMerchants']['max'],
                )),
            ),
            'label' => 'Max Posts',
        ));
        $this->addElement('text', 'option_max_words', array(
            'required' => TRUE,
            'filters' => array(
                'Digits',
            ),
            'validators' => array(
                'Digits',
                array('Between', TRUE, array(
                    'min' => $constraints['textMaxWords']['min'],
                    'max' => $constraints['textMaxWords']['max'],
                )),
            ),
            'label' => 'Max Words'
        ));
        $this->addElement('textarea', 'search_include_keywords', array(
            'filters' => array(
                new Pumka_Filter_KeywordList,
            ),
            'label' => 'Keywords',
        ));
        $this->addElement('textarea', 'search_exclude_keywords', array(
            'filters' => array(
                new Pumka_Filter_KeywordList,
            ),
            'label' => 'Negative Keywords',
        ));
        $this->addElement('select', 'search_language', array(
            'multiOptions' => $multiOptions['searchLanguage'],
            'filters' => array(
                'Null',
            ),
            'label' => 'Language',
        ));
        $this->addElement('select', 'sort_order', array(
            'required' => FALSE,
            'multiOptions' => $multiOptions['sortOrder'],
            'filters' => array(
                'Null',
            ),
            'label' => 'Sort results by',
        ));
        $this->addElement('select', 'sort_direction', array(
            'multiOptions' => $multiOptions['sortDirection'],
            'filters' => array(
                'Null',
            ),
            'label' => 'Sort Direction',
        ));
    }

    public function isValid($data)
    {
        $action = isset($data['action'])? $data['action'] : '';

        if ('add_selected' == $action) {
            $items = $_SESSION[Cbc_WpPlugin::PREFIX . '_preview_items'];
            if (!is_array($items)) {
                throw new Exception('Preview items not found');
            }
            $multiOptions = array();
            foreach ($items as $key=>$item) {
                if ($item['published']) {
                    continue;
                }

                $mutliOptions[$key] = $item['affiliateNetwork-title'];
            }

            $this->addElement('multiCheckbox', 'select_items', array(
                'label' => 'Add Selected',
                'required' => TRUE,
                'multiOptions' => $mutliOptions,
                'validators' => array(
                    array('NotEmpty', TRUE,
                        array(
                            'messages' => array(
                                'isEmpty' => "At least one item must be selected",
                            ),
                        ),
                    ),
                ),
                'help' => FALSE,
            ));
        }

        $core = Cbc_WpModule_Application::getModel('CoreApp');
        $initDefaults = $core->getDefaultValues();

        if (!isset($data['posts_per_week'])) {
            $options = Cbc_WpModule_Application::getModel('Options');
            $data['posts_per_week'] = $options
                ->getOption('posts_per_week', $initDefaults['postsPerWeek']);
        }

        $result = parent::isValid($data);

        if ($result) {
            // If sort order is Keyword Relevance but no keywords set
            if ('' == $this->getValue('sort_order') && '' == $this->getValue('search_include_keywords')) {
                $result = FALSE;
                $this->getElement('search_include_keywords')->addError(
                    'Keywords are required if sort order is keyword relevance');
            }
        }

        $this->_errorsExist = !$result;
        return $result;
    }

    public function setClickbankCategories(array $categories)
    {
        /**
        * @var Cbc_WpModule_Model_Options
        */
        $options = Cbc_WpModule_Application::getModel('Options');

        $categoryElement = $this->getElement('clickbank_category');
        foreach ($categories as $category) {
            $categoryElement->addMultiOption($category['id'], $category['label']);

            $name = "clickbank_subcategory_{$category['id']}";
            $this->addElement('select', $name, array(
                'filters' => array(
                    'Null',
                ),
                'label' => $category['label'],
            ));
            $subcategoryElement = $this->getElement($name);
            foreach ($category['children'] as $subcategory) {
                $subcategoryElement->addMultiOption($subcategory['id'], $subcategory['label']);
            }
        }

        $categoryElement->setAttrib('disabled', FALSE);

        foreach ($categories as $id=>$category) {
            $name = "clickbank_subcategory_{$id}";
            $this->setDefault($name,
                $options->getOption($name));
        }

        $this->_clickbankCategories = $categories;
    }

    public function getClickbankCategories()
    {
        return $this->_clickbankCategories;
    }

    public function loadDefaults()
    {
        /**
        * @var Cbc_WpModule_Model_Options
        */
        $options = Cbc_WpModule_Application::getModel('Options');

        /**
        * @var Cbc_WpModule_Model_CoreApp
        */
        $core = Cbc_WpModule_Application::getModel('CoreApp');
        $initDefaults = $core->getDefaultValues();

        $this->setDefault('clickbank_id',
            $options->getOption('clickbank_id'));
        $this->setDefault('clickbank_password',
            $options->getOption('clickbank_password'));
        $this->setDefault('tracking_id',
            $options->getOption('tracking_id'));
        $this->setDefault('clickbank_category',
            $options->getOption('clickbank_category'));
        $this->setDefault('posts_per_week',
            $options->getOption('posts_per_week', $initDefaults['postsPerWeek']));
        $this->setDefault('publish_type',
            $options->getOption('publish_type', $initDefaults['publishType']));
        $postCategories = $options->getOption('post_category');
        $postCategories = empty($postCategories)
            ? array()
            : explode(',', $postCategories);
        $this->setDefault('post_category', $postCategories);

        /*$this->setDefault('post_template',
            $options->getOption('post_template',
            "[description]\n\n[read more]"));  */

        $this->setDefault('sort_order',
            $options->getOption('sort_order', $initDefaults['sortOrder']));
        $this->setDefault('sort_direction',
            $options->getOption('sort_direction', $initDefaults['sortDirection']));
        $this->setDefault('search_language',
            $options->getOption('search_language', $initDefaults['searchLanguage']));
        $this->setDefault('search_include_keywords',
            $options->getOption('search_include_keywords'));
        $this->setDefault('search_exclude_keywords',
            $options->getOption('search_exclude_keywords'));
        $this->setDefault('option_max_posts',
            $options->getOption('option_max_posts', $initDefaults['maxMerchants']));
        $this->setDefault('option_max_words',
            $options->getOption('option_max_words', $initDefaults['textMaxWords']));
    }

    public function saveDefaults()
    {
        /**
        * @var Cbc_WpModule_Model_Options
        */
        $options = Cbc_WpModule_Application::getModel('Options');

        $category = $this->getValue('clickbank_category');
        $options->setOption('clickbank_category', $category);
        if ($category) {
            $name = "clickbank_subcategory_{$category}";
            $options->setOption($name, $this->getValue($name));
        }
        $options->setOption('clickbank_id',
            $this->getValue('clickbank_id'));
        $options->setOption('clickbank_password',
            $this->getValue('clickbank_password'));
        $options->setOption('tracking_id',
            $this->getValue('tracking_id'));
        $options->setOption('posts_per_week',
            $this->getValue('posts_per_week'));
        $options->setOption('publish_type',
            $this->getValue('publish_type'));
        $options->setOption('post_category', implode(',',
            (array)$this->getValue('post_category')));
        $options->setOption('search_include_keywords',
            $this->getValue('search_include_keywords'));
        $options->setOption('search_exclude_keywords',
            $this->getValue('search_exclude_keywords'));
        $options->setOption('sort_order',
            $this->getValue('sort_order'));
        $options->setOption('sort_direction',
            $this->getValue('sort_direction'));
        $options->setOption('search_language',
            $this->getValue('search_language'));
        $options->setOption('option_max_posts',
            $this->getValue('option_max_posts'));
        $options->setOption('option_max_words',
            $this->getValue('option_max_words'));
    }

    public function getSubcategory()
    {
        $category = $this->getValue('clickbank_category');
        if ($category) {
            $name = "clickbank_subcategory_{$category}";
            return $this->getValue($name);
        }
    }

    public function hasCredentials()
    {
        foreach ($this->_credentialFields as $field) {
            if ('' == $this->getValue($field)) {
                return FALSE;
            }
        }

        return TRUE;
    }

    public function extractCredentials($data = NULL)
    {
        $result = array();

        if (is_null($data) && !$this->hasCredentials()) {
            return NULL;
        }

        foreach ($this->_credentialFields as $field) {
            if (!is_null($data)) {
                if (!isset($data[$field])) {
                    return NULL;
                }
                $value = $data[$field];
            } else {
                $value = $this->getValue($field);
            }

            $result[$field] = $value;
        }

        return $result;
    }
}