<?php
require_once 'CbcZend/Form.php';

require_once 'Pumka/Filter/StripSlashes.php';

class Cbc_WpModule_Form_Options extends CbcZend_Form
{
    protected $_clickbankCategories = array();

    /**
    * @var Cbc_WpModule_Model_Options
    */
    protected $_optionsModel;

    /**
    * @var Cbc_WpModule_Model_Template
    */
    protected $_templateModel;

    public function init()
    {
        $this->_optionsModel = Cbc_WpModule_Application::getModel('Options');
        $this->_templateModel = Cbc_WpModule_Application::getModel('Template');

        $this->setDisableTranslator(TRUE);

        $this->addElement('radio', 'eye_candy_type', array(
            'required' => TRUE,
            'multiOptions' => array(
                'button' => 'Button',
                'thumbnail' => 'Site Thumbnail',
            ),
            'label' => 'Eye candy to use',
            'value' => $this->_templateModel->getOption('eye_candy_type'),
        ));

        $this->addElement('file', 'button_image', array(
            'validators' => array(
                //'IsImage',
                array('Count', TRUE, 1),
                array('Extension', TRUE, array('jpg', 'jpeg', 'png', 'gif')),
                array('FilesSize', TRUE, array('max' => '1MB')),
                //array('ImageSize', TRUE, array('maxheight' => 400, 'maxwidth' => 1024)),
            ),
            'accept' => 'image/jpeg,image/png,image/gif',
            'ignore' => TRUE,
            'label' => 'Button image',
        ));

        $this->addElement('checkbox', 'button_image-reset', array(
            'label' => 'Reset to default',
        ));

        $this->addElement('text', 'button_text', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Button text',
            'value' => $this->_templateModel->getOption('button_text'),
        ));

        $this->addElement('text', 'thumbnail_text', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
            ),
            'label' => 'Thumbnail text',
            'value' => $this->_templateModel->getOption('thumbnail_text'),
        ));

        $this->addElement('select', 'tsize', array(
            'required' => TRUE,
            'multiOptions' => array(
                't' => 'Tiny (90x68)',
                's' => 'Small (120x90)',
                'm' => 'Medium (200x150)',
                'l' => 'Large (400x300)',
                'x' => 'Extra Large (480x360)',
            ),
            'label' => 'Site thumbnail dimensions',
            'value' => $this->_templateModel->getOption('tsize'),
        ));

        $this->addElement('select', 'eye_candy_align', array(
            'required' => TRUE,
            'multiOptions' => array(
                'left' => 'Left',
                'right' => 'Right',
            ),
            'label' => 'Eye candy align',
            'value' => $this->_templateModel->getOption('eye_candy_align'),
        ));

        // Link settings
        $this->addElement('text', 'clickbank_id', array(
            'required' => TRUE,
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                'StringToLower',
            ),
            'validators' => array(
                array('Regex', TRUE, array(
                    'pattern' => '~^[a-z0-9]+$~u',
                    'messages' => array(
                        'regexNotMatch' => "'%value%' contains invalid characters",
                    ),
                )),
            ),
            'label' => 'ClickBank ID',
            'value' => $this->_optionsModel->getOption('clickbank_id'),
        ));

        $this->addElement('text', 'clickbank_password', array(
            'required' => TRUE,
            'filters' => array(
                new Pumka_Filter_StripSlashes,
                'StripNewlines',
            ),
            'label' => 'ClickBank Password',
            'value' => $this->_optionsModel->getOption('clickbank_password'),
        ));

        $this->addElement('text', 'tracking_id', array(
            'filters' => array(
                'StripNewlines',
                'StringTrim',
                array('Null', array('string')),
            ),
            'validators' => array(
                array('StringLength', TRUE, array(
                    'max' => 24,
                )),
            ),
            'label' => 'Append this tracking ID to my affiliate links',
            'value' => $this->_optionsModel->getOption('tracking_id'),
        ));
    }

    public function isValid($data)
    {
        if (!isset($data['eye_candy_type']) || 'button' == $data['eye_candy_type']) {
            $this->getElement('tsize')->setRequired(FALSE);
        }

        return parent::isValid($data);
    }

    public function saveValues()
    {
        $this->_saveValue('eye_candy_type');
        $this->_saveValue('button_text');
        $this->_saveValue('thumbnail_text');
        $this->_saveValue('tsize');
        $this->_saveValue('eye_candy_align');

        if ($this->getValue('button_image-reset')) {
            $this->_templateModel->resetButtonImage();
        } else {
            /**
            * @var CbcZend_Form_Element_File
            */
            $upload = $this->getElement('button_image');
            if ($upload->isUploaded()) {
                try {
                    $this->_templateModel->uploadButtonImage($upload);
                } catch (Cbc_WpModule_Model_Template_Exception $e) {
                    throw new Cbc_WpModule_Form_Options_Exception(
                        $e->getMessage(), 'button_image');
                }
            }
        }

        $this->_saveValue('clickbank_id');
        $this->_saveValue('clickbank_password');
        $this->_saveValue('tracking_id');
    }

    protected function _saveValue($name)
    {
        $this->_optionsModel->setOption($name,
            $this->getValue($name));
    }
}

class Cbc_WpModule_Form_Options_Exception extends Exception
{
    protected $_element;

    public function __construct($message, $element=NULL)
    {
        parent::__construct($message);

        if (!is_null($element)) {
            $this->_element  = $element;
        }
    }

    public function getElement()
    {
        return $this->_element;
    }
}