<?php
class Cbc_WpModule_Model_CoreApp
{
    protected $_app;
    protected $_sessionNamespace = 'anc';

    protected $_defaultValues = array(
        'postsPerWeek' => '7',
        'publishType' => 'schedule',
        'maxMerchants' => 100,
        'textMaxWords' => 300,
        'searchLanguage' => 'EN',
        'sortOrder' => 'GRAVITY',
        'sortDirection' => 'DESC',
    );

    protected $_constraints = array(
        'maxMerchants' => array(
            'min' => 10,
            'max' => 500,
        ),
        'textMaxWords' => array(
            'min' => 100,
            'max' => 1000,
        ),
    );

    protected $_scheduleOptions = array(
        '1' => array(
            'min' => 0,
            'avg' => 0,
            'max' => 1,
            'weekly' => 1,
        ),
        '3' => array(
            'min' => 0,
            'avg' => 0,
            'max' => 1,
            'weekly' => 3,
        ),
        '7' => array(
            'min' => 0,
            'avg' => 1,
            'max' => 2,
            'weekly' => 7,
        ),
        '15' => array(
            'min' => 0,
            'avg' => 2,
            'max' => 4,
            'weekly' => 15,
        ),
        '30' => array(
            'min' => 2,
            'avg' => 4,
            'max' => 6,
            'weekly' => 30,
        ),
        '100' => array(
            'min' => 12,
            'avg' => 14,
            'max' => 16,
            'weekly' => 100,
        ),
    );

    protected $_multiOptions = array(
        'searchLanguage' => array(
            'EN' => 'English',
            'FR' => 'French',
            'DE' => 'German',
            'IT' => 'Italian',
            'PT' => 'Portuguese',
            'ES' => 'Spanish',
        ),
        'sortOrder' => array(
            'GRAVITY' => 'Gravity',
            'POPULARITY' => 'Popularity',
            'AVERAGE_EARNINGS_PER_SALE' => 'Avg $/sale',
            '' => 'Keyword Relevance',
        ),
        'sortDirection' => array(
            'DESC' => 'High to Low',
            'ASC' => 'Low to High',
        ),
    );

    protected $_categoryCacheLifetime = 86400;

    public function getCategories(array $credentials)
    {
        // Cache categories to transient option
        $transientOptions = Cbc_WpModule_Application::getModel('TransientOptions');
        $result = $transientOptions->getValue('clickbank_categories');
        if ($result) {
            return $result;
        }

        $adapter = $this->getApp()->getAdapter();
        $adapter->setOptions(array(
            'affiliateId' => $credentials['clickbank_id'],
            'password' => $credentials['clickbank_password'],
        ));

        try {
            $result = $adapter->getCategories();
        } catch (Anc_Library_Plugin_Exception $e) {
            throw new Cbc_WpModule_Model_CoreAppException($e->getMessage());
        }

        // Cache categories to transient option
        $transientOptions->setValue('clickbank_categories', $result,
            $this->_categoryCacheLifetime);

        return $result;
    }

    public function setup($options)
    {
        $app = $this->getApp();

        if (!isset($options['postsPerWeek'])) {
            throw new Cbc_WpModule_Model_CoreAppException('Missing postsPerWeek option');
        }
        $value = $options['postsPerWeek'];
        if (!isset($this->_scheduleOptions[$value])) {
            throw new Cbc_WpModule_Model_CoreAppException('Invalid postsPerWeek option');
        }
        $options['publisher']['scheduleSettings'] = $this->_scheduleOptions[$value];
        $options['publisher']['publishType'] = $options['publishType'];

        //some display options:
        $template = Cbc_WpModule_Application::getModel('Template');
        $options['main']['pingThumbs'] = ('thumbnail' == $template->getOption('eye_candy_type'));
        $options['main']['thumbSize'] = $template->getOption('tsize');

        $app->setOptions($options['main']);
        $app->getAdapter()->setOptions($options['adapter']);
        $app->getScraper()->setOptions($options['scraper']);
        $app->getPublisher()->setOptions($options['publisher']);
    }

    public function generate()
    {
        @session_start();
        unset($_SESSION[$this->_sessionNamespace]);

        return $this->getApp()->generate();
    }

    public function isActive()
    {
        if (!(isset($_SESSION))) @session_start();

        if ((!isset($_SESSION[$this->_sessionNamespace]))
            || (isset($_SESSION[$this->_sessionNamespace]['command'])
            && 'terminate' == $_SESSION[$this->_sessionNamespace]['command']))
        {
            $result = FALSE;
        } else {
            $result = TRUE;
        }

        return $result;
    }

    public function getOutput()
    {
        @session_start();
        return @$_SESSION[$this->_sessionNamespace];
        session_write_close();
    }

    public function sendCommand($command)
    {
        @session_start();
        $_SESSION[$this->_sessionNamespace]['command'] = $command;
        session_write_close();
    }

    public function terminate()
    {
        $this->sendCommand('terminate');
    }

    public function clearOutput()
    {
        @session_start();
        unset($_SESSION[$this->_sessionNamespace]);
        session_write_close();
    }

    //
    // Preview
    //

    public function preview($options=array())
    {
        $app = $this->getApp();

        $app->setOptions($options['main']);
        $app->getAdapter()->setOptions($options['adapter']);

        return $app->preview();
    }

    //
    // Getters & setters
    //

    public function getApp()
    {
        if (!isset($this->_app)) {
            require_once realpath(CBC_WPMODULE_PATH . '/../app/Application.php');
            $this->_app = Anc_Application::app();
        }

        return $this->_app;
    }

    public function getDefaultValues()
    {
        return $this->_defaultValues;
    }

    public function getConstraints()
    {
        return $this->_constraints;
    }

    public function getMultiOptions()
    {
        $result = $this->_multiOptions;

        // Add schedule options
        $options = array();
        foreach (array_keys($this->_scheduleOptions) as $option) {
            $options[$option] = $option;
        }
        $result['postsPerWeek'] = $options;

        return $result;
    }
}

class Cbc_WpModule_Model_CoreAppException extends Exception {}
