<?php
class Cbc_WpModule_Model_Template
{
    /**
    * @var Cbc_WpModule_Model_Options
    */
    protected $_optionsModel;

    protected $_defaults = array(
        'eye_candy_type' => 'thumbnail',
        'button_text' => 'Visit Site',
        'thumbnail_text' => 'Click Image To Visit Site',
        'tsize' => 'm',
        'eye_candy_align' => 'right',
    );

    protected $_thumbnailSizes = array(
        't' => array(90, 68),
        's' => array(120, 90),
        'm' => array(200, 150),
        'l' => array(400, 300),
        'x' => array(480, 360),
    );

    //How long to keep thumb status
    protected $_thumbStatusCacheInterval = 864000;

    public function __construct()
    {
        $this->_optionsModel = Cbc_WpModule_Application::getModel('Options');
    }

    public function prepareTemplateVars(array &$vars)
    {
        $vars['eyeCandy'] = (object)($this->getEyeCandy($vars));
        $vars['affiliateUrl'] = $this->_generateAffiliateUrl($vars);
    }

    public function getOption($name)
    {
        return $this->_optionsModel
        ->getOption($name, isset($this->_defaults[$name])
            ? $this->_defaults[$name]
            : NULL
        );
    }

    public function getEyeCandy($vars)
    {
        $type = $this->getOption('eye_candy_type');

        if ('thumbnail' == $type) {
            $result = $this->_getEyeCandyThumbnail($vars);
            $result['bottomText'] = $this->getOption('thumbnail_text');
            if (!$result) {
                $type = 'button';
            }
        }
        if ('button' == $type || '' == $type) {
            $result = $this->_getEyeCandyButton();
        }

        $result['type'] = $type;
        $result['align'] = $this->getOption('eye_candy_align');

        return $result;
    }

    protected function _generateAffiliateUrl(array $vars)
    {
        $post = $vars['post'];

        global $wp_rewrite;

        $path = array(untrailingslashit(site_url()));

        if (!$wp_rewrite->using_mod_rewrite_permalinks()) {
            $path[] = $wp_rewrite->index;
        }

        $path[] = Cbc_WpModule_Application::getModel('Options')
            ->getOption('cloak_dir', Cbc_WpPlugin::DEFAULT_CLOAK_DIR);
        $path[] = $post->post_name;

        $result = implode('/', $path);

        return $result;
    }

    protected function _getEyeCandyButton()
    {
        $result = $this->getButtonImage();
        $result['text'] = $this->getOption('button_text');

        return $result;
    }

    protected function _getEyeCandyThumbnail($vars)
    {
        $result = array();

        $thumbSize = $this->getOption('tsize');

        require_once 'Pumka/Thumbs/PagePeeker.php';
        $thumb = new Pumka_Thumbs_PagePeeker;
        $thumb->setOptions(array(
                'size'  => $thumbSize,
                'url'   => $vars['merchant_url'],
            ));

        if (!isset($vars['thumbnail_timestamp'])
            || (time() - $vars['thumbnail_timestamp']) > $this->_thumbStatusCacheInterval
        ) {
            if ($thumb->isImageReady()) {
                Cbc_WpModule_Application::getModel('PostMeta')->setPostId($vars['post_id'])
                ->setValue('thumbnail_timestamp', time());
            } else {
                $thumb->downloadImage();
                return $result;
            }
        }

        $result = array(
            'url' => $thumb->getThumbUrl(),
        );

        $sizeInfo = $this->_thumbnailSizes[$thumbSize];
        $result['width'] = $sizeInfo[0];
        $result['height'] = $sizeInfo[1];

        return $result;
    }

    public function uploadButtonImage(CbcZend_Form_Element_File $upload)
    {
        $fieldName = $upload->getName();

        $info = $upload->getTransferAdapter()->getFileInfo();
        $info = $info[$fieldName];
        @$sizeInfo = getimagesize($info['tmp_name']);
        if (FALSE === $sizeInfo) {
            throw new Cbc_WpModule_Model_Template_Exception('Invalid image file');
        }

        $fileInfo = wp_handle_upload($_FILES[$fieldName],
            array('test_form' => FALSE));
        if (isset($fileInfo['error'])) {
            throw new Cbc_WpModule_Model_Template_Exception(
                "Error uploading file: {$fileInfo['error']}");
        }

        // Add size data
        $fileInfo['width'] = $sizeInfo[0];
        $fileInfo['height'] = $sizeInfo[1];

        //Save option
        $this->_optionsModel->setOption('button_image', $fileInfo);
    }

    public function resetButtonImage()
    {
        $this->_optionsModel->unsetOption('button_image');
    }

    public function getButtonImage()
    {
        $result = $this->_optionsModel->getOption('button_image');
        if (!$result
            || !is_array($result)
            || !isset($result['file'])
            || !isset($result['url'])
            || !isset($result['width'])
            || !isset($result['height'])
        ) {
            $result = $this->_getDefaultButtonImage();
        }

        return $result;
    }

    protected function _getDefaultButtonImage()
    {
        $plugin = Cbc_WpPlugin::getInstance();
        $result = array(
            'file' => $plugin->getPluginPath() . '/template/images/visit-button.png',
            'url' => $plugin->getPluginBaseUrl() . '/template/images/visit-button.png',
            'width' => 300,
            'height' => 80,
            'type' => 'image/png',
            'default' => TRUE,
        );

        return $result;
    }

    //
    // Virtual meta
    //
    public function getThumbnail($vars)
    {
        return $this->_getEyeCandyThumbnail($vars);
    }

    public function getAffiliateUrl($vars)
    {
        return $this->_generateAffiliateUrl($vars);
    }

    //
    // Getters and setters
    //
    public function getDefault($key=NULL)
    {
        if (is_null($key)) {
            return $this->_defaults;
        }

        return @$this->_defaults[$key];
    }


}

class Cbc_WpModule_Model_Template_Exception extends Exception
{

}